# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016

import warnings

import mock
import numpy as np
import pandas as pd
import pytest

from tsfresh.feature_selection.relevance import (
    calculate_relevance_table,
    combine_relevance_tables,
    get_feature_type,
    infer_ml_task,
)


class TestInferMLTask:
    def test_infers_classification_for_integer_target(self):
        y = pd.Series([1, 2, 3])
        assert "classification" == infer_ml_task(y)

    def test_infers_classification_for_boolean_target(self):
        y = pd.Series([True, False, False])
        assert "classification" == infer_ml_task(y)

    def test_infers_classification_for_object_target(self):
        y = pd.Series(["high", "low"])
        assert "classification" == infer_ml_task(y)

    def test_infers_regression_for_float_target(self):
        y = pd.Series([1.0, 1.5, 1.7])
        assert "regression" == infer_ml_task(y)


class TestCalculateRelevanceTable:
    @pytest.fixture()
    def y_binary(self):
        return pd.Series([0, 1, 1])

    @pytest.fixture()
    def y_real(self):
        return pd.Series([0.1, 0.2, 0.1])

    @pytest.fixture()
    def y_multi(self):
        y0 = np.zeros(100)
        y1 = np.repeat(1, 100)
        y2 = np.repeat(2, 100)
        return pd.Series(np.uint8(np.concatenate([y0, y1, y2])))

    @pytest.fixture()
    def X(self):
        df = pd.DataFrame()
        df["feature_binary"] = [1, 1, 0]
        df["feature_real"] = [0.1, 0.2, 0.3]
        return df

    def test_restrict_ml_task_options(self, X, y_binary):
        with pytest.raises(ValueError):
            calculate_relevance_table(X, y_binary, ml_task="some_other_task")

    def test_constant_feature_irrelevant(self, y_binary):
        X = pd.DataFrame([1, 1, 1], columns=["feature_binary"])

        relevance_table = calculate_relevance_table(X, y_binary)
        assert "feature_binary" == relevance_table.index[0]
        assert "constant" == relevance_table.type[0]
        assert np.isnan(relevance_table.p_value[0])
        assert not relevance_table.relevant[0]

    @mock.patch("tsfresh.feature_selection.relevance.target_binary_feature_real_test")
    @mock.patch("tsfresh.feature_selection.relevance.target_binary_feature_binary_test")
    def test_target_binary_calls_correct_tests(
        self,
        significance_test_feature_binary_mock,
        significance_test_feature_real_mock,
        X,
        y_binary,
    ):
        significance_test_feature_binary_mock.return_value = 0.5
        significance_test_feature_real_mock.return_value = 0.7
        relevance_table = calculate_relevance_table(X, y_binary, n_jobs=0)

        assert 0.5 == relevance_table.loc["feature_binary"].p_value
        assert 0.7 == relevance_table.loc["feature_real"].p_value
        assert 2 == significance_test_feature_binary_mock.call_count
        assert 2 == significance_test_feature_real_mock.call_count

    @mock.patch("tsfresh.feature_selection.relevance.target_real_feature_real_test")
    @mock.patch("tsfresh.feature_selection.relevance.target_real_feature_binary_test")
    def test_target_real_calls_correct_tests(
        self,
        significance_test_feature_binary_mock,
        significance_test_feature_real_mock,
        X,
        y_real,
    ):
        significance_test_feature_binary_mock.return_value = 0.5
        significance_test_feature_real_mock.return_value = 0.7

        relevance_table = calculate_relevance_table(X, y_real, n_jobs=0)

        assert 0.5 == relevance_table.loc["feature_binary"].p_value
        assert 0.7 == relevance_table.loc["feature_real"].p_value

        assert significance_test_feature_binary_mock.call_count == 1
        pd.testing.assert_series_equal(
            significance_test_feature_binary_mock.call_args[0][0], X["feature_binary"]
        )
        pd.testing.assert_series_equal(
            significance_test_feature_binary_mock.call_args[1]["y"], y_real
        )

        assert significance_test_feature_real_mock.call_count == 1
        pd.testing.assert_series_equal(
            significance_test_feature_real_mock.call_args[0][0], X["feature_real"]
        )
        pd.testing.assert_series_equal(
            significance_test_feature_real_mock.call_args[1]["y"], y_real
        )

    @mock.patch("tsfresh.feature_selection.relevance.target_real_feature_real_test")
    @mock.patch("tsfresh.feature_selection.relevance.target_real_feature_binary_test")
    def test_warning_for_no_relevant_feature(
        self,
        significance_test_feature_binary_mock,
        significance_test_feature_real_mock,
        X,
        y_real,
    ):
        significance_test_feature_binary_mock.return_value = 0.95
        significance_test_feature_real_mock.return_value = 0.95

        with pytest.warns(RuntimeWarning) as record:
            _ = calculate_relevance_table(
                X, y_real, n_jobs=0, ml_task="regression", show_warnings=True
            )
            assert len(record) >= 1
            assert (
                "No feature was found relevant for regression for fdr level = 0.05 (which corresponds "
                "to the maximal percentage of irrelevant features, consider using an higher fdr level "
                "or add other features."
            ) in [str(warning.message) for warning in record]

    def test_multiclass_requires_classification(self, X, y_real):
        with pytest.raises(AssertionError):
            calculate_relevance_table(X, y_real, multiclass=True, ml_task="regression")

    def test_multiclass_n_significant_error(self, X, y_binary):
        with pytest.raises(AssertionError):
            calculate_relevance_table(
                X, y_binary, multiclass=True, n_significant=3, ml_task="classification"
            )

    def test_multiclass_relevance_table_columns(self, X, y_binary):
        y = y_binary.copy()
        y[2] = 2
        relevance_table = calculate_relevance_table(X, y, multiclass=True)

        assert len(relevance_table.columns) == 10

    def test_multiclass_correct_features_relevant(self, y_multi):
        X_multi = pd.DataFrame()
        X_multi["relevant_0"] = np.concatenate([np.zeros(298), np.array([0.01, -0.01])])
        X_multi["relevant_3"] = X_multi["relevant_0"].copy()
        X_multi["relevant_3"][y_multi == 0] = np.random.uniform(2, 3, 100)
        X_multi["relevant_2"] = X_multi["relevant_3"].copy()
        X_multi["relevant_2"][y_multi == 1] = np.random.uniform(-2, -1, 100)

        relevance_table = calculate_relevance_table(
            X_multi, y_multi, multiclass=True, ml_task="classification", n_significant=3
        )
        assert relevance_table.loc["relevant_3", "relevant"]
        assert not relevance_table.loc["relevant_2", "relevant"]
        assert not relevance_table.loc["relevant_0", "relevant"]

        # the distributions of all 3 classes under a one vs. rest scheme will be separated enough for
        # this feature to be relevant for predicting 3 classes
        assert relevance_table.loc["relevant_3", "n_significant"] == 3

        # due to the distribution of this feature where y_multi == 0 being contained inside the range of
        # y_multi != 0 it will not pass the Mann-Whitney U test under a one vs. rest scheme for that class
        assert relevance_table.loc["relevant_2", "n_significant"] == 2
        assert relevance_table.loc["relevant_0", "n_significant"] == 0


class TestCombineRelevanceTables:
    @pytest.fixture()
    def relevance_table(self):
        relevance_table = pd.DataFrame(
            index=pd.Series(["f1", "f2", "f3", "f4"], name="feature")
        )
        relevance_table["relevant"] = [True, False, True, False]
        relevance_table["type"] = ["real"] * 4
        relevance_table["p_value"] = [0.1, 0.2, 0.3, 0.4]
        return relevance_table

    def test_disjuncts_relevance(self, relevance_table):
        relevance_table_2 = relevance_table.copy()
        relevance_table_2.relevant = [False, True, True, False]
        result = combine_relevance_tables([relevance_table, relevance_table_2])

        assert ([True, True, True, False] == result.relevant).all()

    def test_respects_index(self, relevance_table):
        relevance_table_2 = relevance_table.copy()
        relevance_table_2.reindex(reversed(relevance_table.index))

        result = combine_relevance_tables([relevance_table, relevance_table_2])

        assert ([True, False, True, False] == result.relevant).all()

    def test_aggregates_p_value(self, relevance_table):
        relevance_table_2 = relevance_table.copy()
        relevance_table_2.p_value = [0.2, 0.1, 0.4, 0.3]
        result = combine_relevance_tables([relevance_table, relevance_table_2])

        assert (np.array([0.1, 0.1, 0.3, 0.3]) == result.p_value).all()


class TestGetFeatureType:
    def test_binary(self):
        feature = pd.Series([0.0, 1.0, 1.0])
        assert "binary" == get_feature_type(feature)

    def test_constant(self):
        feature = pd.Series([0.0, 0.0, 0.0])
        assert "constant" == get_feature_type(feature)

    def test_real(self):
        feature = pd.Series([0.0, 1.0, 2.0])
        assert "real" == get_feature_type(feature)



# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016
import warnings
from unittest import TestCase

import numpy as np
import pandas as pd
from pandas.testing import assert_frame_equal, assert_series_equal

from tests.fixtures import warning_free
from tsfresh import extract_relevant_features
from tsfresh.feature_extraction.settings import MinimalFCParameters
from tsfresh.utilities import dataframe_functions


class RollingTestCase(TestCase):
    def test_with_wrong_input(self):
        test_df = pd.DataFrame(
            {
                "id": [0, 0],
                "kind": ["a", "b"],
                "value": [3, 3],
                "sort": [np.nan, np.nan],
            }
        )
        self.assertRaises(
            ValueError,
            dataframe_functions.roll_time_series,
            df_or_dict=test_df,
            column_id="id",
            column_sort="sort",
            column_kind="kind",
            rolling_direction=1,
            n_jobs=0,
        )

        test_df = pd.DataFrame(
            {"id": [0, 0], "kind": ["a", "b"], "value": [3, 3], "sort": [1, 1]}
        )
        self.assertRaises(
            AttributeError,
            dataframe_functions.roll_time_series,
            df_or_dict=test_df,
            column_id="strange_id",
            column_sort="sort",
            column_kind="kind",
            rolling_direction=1,
            n_jobs=0,
        )

        self.assertRaises(
            ValueError,
            dataframe_functions.roll_time_series,
            df_or_dict=test_df,
            column_id=None,
            column_sort="sort",
            column_kind="kind",
            rolling_direction=1,
            n_jobs=0,
        )

        test_df = {"a": pd.DataFrame([{"id": 0}])}
        self.assertRaises(
            ValueError,
            dataframe_functions.roll_time_series,
            df_or_dict=test_df,
            column_id="id",
            column_sort=None,
            column_kind="kind",
            rolling_direction=1,
            n_jobs=0,
        )

        self.assertRaises(
            ValueError,
            dataframe_functions.roll_time_series,
            df_or_dict=test_df,
            column_id=None,
            column_sort=None,
            column_kind="kind",
            rolling_direction=1,
            n_jobs=0,
        )

        self.assertRaises(
            ValueError,
            dataframe_functions.roll_time_series,
            df_or_dict=test_df,
            column_id="id",
            column_sort=None,
            column_kind=None,
            rolling_direction=0,
            n_jobs=0,
        )

        self.assertRaises(
            ValueError,
            dataframe_functions.roll_time_series,
            df_or_dict=test_df,
            column_id=None,
            column_sort=None,
            column_kind=None,
            rolling_direction=0,
            n_jobs=0,
        )

        test_df = pd.DataFrame(
            {"id": [0, 0], "kind": ["a", "b"], "value": [3, 3], "sort": [1, 1]}
        )
        self.assertRaises(
            ValueError,
            dataframe_functions.roll_time_series,
            df_or_dict=test_df,
            column_id="id",
            column_kind="kind",
            column_sort="sort",
            max_timeshift=0,
            rolling_direction=1,
            n_jobs=0,
        )

        self.assertRaises(
            ValueError,
            dataframe_functions.roll_time_series,
            df_or_dict=test_df,
            column_id="id",
            column_kind="kind",
            column_sort="sort",
            min_timeshift=-1,
            rolling_direction=1,
            n_jobs=0,
        )

    def test_assert_single_row(self):
        test_df = pd.DataFrame([{"id": np.nan, "kind": "a", "value": 3, "sort": 1}])
        self.assertRaises(
            ValueError,
            dataframe_functions.roll_time_series,
            df_or_dict=test_df,
            column_id="id",
            column_sort="sort",
            column_kind="kind",
            rolling_direction=1,
            n_jobs=0,
        )

    def test_positive_rolling(self):
        first_class = pd.DataFrame(
            {"a": [1, 2, 3, 4], "b": [5, 6, 7, 8], "time": range(4)}
        )
        second_class = pd.DataFrame(
            {"a": [10, 11], "b": [12, 13], "time": range(20, 22)}
        )

        first_class["id"] = 1
        second_class["id"] = 2

        df_full = pd.concat([first_class, second_class], ignore_index=True)

        """ df_full is
            a   b  time  id
        0   1   5     0   1
        1   2   6     1   1
        2   3   7     2   1
        3   4   8     3   1
        4  10  12    20   2
        5  11  13    21   2
        """
        correct_indices = [
            (1, 0),
            (1, 1),
            (1, 1),
            (1, 2),
            (1, 2),
            (1, 2),
            (1, 3),
            (1, 3),
            (1, 3),
            (1, 3),
            (2, 20),
            (2, 21),
            (2, 21),
        ]
        correct_values_a = [
            1.0,
            1.0,
            2.0,
            1.0,
            2.0,
            3.0,
            1.0,
            2.0,
            3.0,
            4.0,
            10.0,
            10.0,
            11.0,
        ]
        correct_values_b = [
            5.0,
            5.0,
            6.0,
            5.0,
            6.0,
            7.0,
            5.0,
            6.0,
            7.0,
            8.0,
            12.0,
            12.0,
            13.0,
        ]

        df = dataframe_functions.roll_time_series(
            df_full,
            column_id="id",
            column_sort="time",
            column_kind=None,
            rolling_direction=1,
            n_jobs=0,
        )

        self.assertListEqual(list(df["id"]), correct_indices)
        self.assertListEqual(list(df["a"].values), correct_values_a)
        self.assertListEqual(list(df["b"].values), correct_values_b)

        df = dataframe_functions.roll_time_series(
            df_full,
            column_id="id",
            column_sort="time",
            column_kind=None,
            rolling_direction=1,
            max_timeshift=4,
            n_jobs=0,
        )

        self.assertListEqual(list(df["id"]), correct_indices)
        self.assertListEqual(list(df["a"].values), correct_values_a)
        self.assertListEqual(list(df["b"].values), correct_values_b)

        df = dataframe_functions.roll_time_series(
            df_full,
            column_id="id",
            column_sort="time",
            column_kind=None,
            rolling_direction=1,
            max_timeshift=2,
            n_jobs=0,
        )
        correct_indices = [
            (1, 0),
            (1, 1),
            (1, 1),
            (1, 2),
            (1, 2),
            (1, 2),
            (1, 3),
            (1, 3),
            (1, 3),
            (2, 20),
            (2, 21),
            (2, 21),
        ]
        correct_values_a = [
            1.0,
            1.0,
            2.0,
            1.0,
            2.0,
            3.0,
            2.0,
            3.0,
            4.0,
            10.0,
            10.0,
            11.0,
        ]
        correct_values_b = [
            5.0,
            5.0,
            6.0,
            5.0,
            6.0,
            7.0,
            6.0,
            7.0,
            8.0,
            12.0,
            12.0,
            13.0,
        ]

        self.assertListEqual(list(df["id"]), correct_indices)
        self.assertListEqual(list(df["a"].values), correct_values_a)
        self.assertListEqual(list(df["b"].values), correct_values_b)

        df = dataframe_functions.roll_time_series(
            df_full,
            column_id="id",
            column_sort="time",
            column_kind=None,
            rolling_direction=1,
            max_timeshift=2,
            min_timeshift=2,
            n_jobs=0,
        )

        correct_indices = [(1, 2), (1, 2), (1, 2), (1, 3), (1, 3), (1, 3)]
        correct_values_a = [1.0, 2.0, 3.0, 2.0, 3.0, 4.0]
        correct_values_b = [5.0, 6.0, 7.0, 6.0, 7.0, 8.0]

        self.assertListEqual(list(df["id"]), correct_indices)
        self.assertListEqual(list(df["a"].values), correct_values_a)
        self.assertListEqual(list(df["b"].values), correct_values_b)

    def test_negative_rolling(self):
        first_class = pd.DataFrame(
            {"a": [1, 2, 3, 4], "b": [5, 6, 7, 8], "time": range(4)}
        )
        second_class = pd.DataFrame(
            {"a": [10, 11], "b": [12, 13], "time": range(20, 22)}
        )

        first_class["id"] = 1
        second_class["id"] = 2

        df_full = pd.concat([first_class, second_class], ignore_index=True)
        """ df_full is
            a   b  time  id
        0   1   5     0   1
        1   2   6     1   1
        2   3   7     2   1
        3   4   8     3   1
        4  10  12    20   2
        5  11  13    21   2
        """

        correct_indices = [
            (1, 0),
            (1, 0),
            (1, 0),
            (1, 0),
            (1, 1),
            (1, 1),
            (1, 1),
            (1, 2),
            (1, 2),
            (1, 3),
            (2, 20),
            (2, 20),
            (2, 21),
        ]
        correct_values_a = [
            1.0,
            2.0,
            3.0,
            4.0,
            2.0,
            3.0,
            4.0,
            3.0,
            4.0,
            4.0,
            10.0,
            11.0,
            11.0,
        ]
        correct_values_b = [
            5.0,
            6.0,
            7.0,
            8.0,
            6.0,
            7.0,
            8.0,
            7.0,
            8.0,
            8.0,
            12.0,
            13.0,
            13.0,
        ]

        df = dataframe_functions.roll_time_series(
            df_full,
            column_id="id",
            column_sort="time",
            column_kind=None,
            rolling_direction=-1,
            n_jobs=0,
        )

        self.assertListEqual(list(df["id"].values), correct_indices)
        self.assertListEqual(list(df["a"].values), correct_values_a)
        self.assertListEqual(list(df["b"].values), correct_values_b)

        df = dataframe_functions.roll_time_series(
            df_full,
            column_id="id",
            column_sort="time",
            column_kind=None,
            rolling_direction=-1,
            max_timeshift=None,
            n_jobs=0,
        )

        self.assertListEqual(list(df["id"].values), correct_indices)
        self.assertListEqual(list(df["a"].values), correct_values_a)
        self.assertListEqual(list(df["b"].values), correct_values_b)

        df = dataframe_functions.roll_time_series(
            df_full,
            column_id="id",
            column_sort="time",
            column_kind=None,
            rolling_direction=-1,
            max_timeshift=1,
            n_jobs=0,
        )

        correct_indices = [
            (1, 0),
            (1, 0),
            (1, 1),
            (1, 1),
            (1, 2),
            (1, 2),
            (1, 3),
            (2, 20),
            (2, 20),
            (2, 21),
        ]
        correct_values_a = [1.0, 2.0, 2.0, 3.0, 3.0, 4.0, 4.0, 10.0, 11.0, 11.0]
        correct_values_b = [5.0, 6.0, 6.0, 7.0, 7.0, 8.0, 8.0, 12.0, 13.0, 13.0]

        self.assertListEqual(list(df["id"].values), correct_indices)
        self.assertListEqual(list(df["a"].values), correct_values_a)
        self.assertListEqual(list(df["b"].values), correct_values_b)

        df = dataframe_functions.roll_time_series(
            df_full,
            column_id="id",
            column_sort="time",
            column_kind=None,
            rolling_direction=-1,
            max_timeshift=2,
            n_jobs=0,
        )

        correct_indices = [
            (1, 0),
            (1, 0),
            (1, 0),
            (1, 1),
            (1, 1),
            (1, 1),
            (1, 2),
            (1, 2),
            (1, 3),
            (2, 20),
            (2, 20),
            (2, 21),
        ]
        correct_values_a = [
            1.0,
            2.0,
            3.0,
            2.0,
            3.0,
            4.0,
            3.0,
            4.0,
            4.0,
            10.0,
            11.0,
            11.0,
        ]
        correct_values_b = [
            5.0,
            6.0,
            7.0,
            6.0,
            7.0,
            8.0,
            7.0,
            8.0,
            8.0,
            12.0,
            13.0,
            13.0,
        ]

        self.assertListEqual(list(df["id"].values), correct_indices)
        self.assertListEqual(list(df["a"].values), correct_values_a)
        self.assertListEqual(list(df["b"].values), correct_values_b)

        df = dataframe_functions.roll_time_series(
            df_full,
            column_id="id",
            column_sort="time",
            column_kind=None,
            rolling_direction=-1,
            max_timeshift=4,
            n_jobs=0,
        )

        correct_indices = [
            (1, 0),
            (1, 0),
            (1, 0),
            (1, 0),
            (1, 1),
            (1, 1),
            (1, 1),
            (1, 2),
            (1, 2),
            (1, 3),
            (2, 20),
            (2, 20),
            (2, 21),
        ]
        correct_values_a = [
            1.0,
            2.0,
            3.0,
            4.0,
            2.0,
            3.0,
            4.0,
            3.0,
            4.0,
            4.0,
            10.0,
            11.0,
            11.0,
        ]
        correct_values_b = [
            5.0,
            6.0,
            7.0,
            8.0,
            6.0,
            7.0,
            8.0,
            7.0,
            8.0,
            8.0,
            12.0,
            13.0,
            13.0,
        ]

        self.assertListEqual(list(df["id"].values), correct_indices)
        self.assertListEqual(list(df["a"].values), correct_values_a)
        self.assertListEqual(list(df["b"].values), correct_values_b)

        df = dataframe_functions.roll_time_series(
            df_full,
            column_id="id",
            column_sort="time",
            column_kind=None,
            rolling_direction=-1,
            min_timeshift=2,
            max_timeshift=3,
            n_jobs=0,
        )

        correct_indices = [(1, 0), (1, 0), (1, 0), (1, 0), (1, 1), (1, 1), (1, 1)]
        correct_values_a = [1.0, 2.0, 3.0, 4.0, 2.0, 3.0, 4.0]
        correct_values_b = [5.0, 6.0, 7.0, 8.0, 6.0, 7.0, 8.0]

        self.assertListEqual(list(df["id"].values), correct_indices)
        self.assertListEqual(list(df["a"].values), correct_values_a)
        self.assertListEqual(list(df["b"].values), correct_values_b)

    def test_rolling_with_larger_shift(self):
        first_class = pd.DataFrame(
            {"a": [1, 2, 3, 4], "b": [5, 6, 7, 8], "time": range(4)}
        )
        second_class = pd.DataFrame(
            {"a": [10, 11], "b": [12, 13], "time": range(20, 22)}
        )

        first_class["id"] = 1
        second_class["id"] = 2

        df_full = pd.concat([first_class, second_class], ignore_index=True)

        """ df_full is
            a   b  time  id
        0   1   5     0   1
        1   2   6     1   1
        2   3   7     2   1
        3   4   8     3   1
        4  10  12    20   2
        5  11  13    21   2
        """
        correct_indices = [
            (1, 1),
            (1, 1),
            (1, 3),
            (1, 3),
            (1, 3),
            (1, 3),
            (2, 21),
            (2, 21),
        ]
        correct_values_a = [1.0, 2.0, 1.0, 2.0, 3.0, 4.0, 10.0, 11.0]
        correct_values_b = [5.0, 6.0, 5.0, 6.0, 7.0, 8.0, 12.0, 13.0]

        df = dataframe_functions.roll_time_series(
            df_full,
            column_id="id",
            column_sort="time",
            column_kind=None,
            rolling_direction=2,
            n_jobs=0,
        )

        self.assertListEqual(list(df["id"]), correct_indices)
        self.assertListEqual(list(df["a"].values), correct_values_a)
        self.assertListEqual(list(df["b"].values), correct_values_b)

        correct_indices = [
            (1, 0),
            (1, 0),
            (1, 0),
            (1, 0),
            (1, 2),
            (1, 2),
            (2, 20),
            (2, 20),
        ]
        correct_values_a = [1.0, 2.0, 3.0, 4.0, 3.0, 4.0, 10.0, 11.0]
        correct_values_b = [5.0, 6.0, 7.0, 8.0, 7.0, 8.0, 12.0, 13.0]

        df = dataframe_functions.roll_time_series(
            df_full,
            column_id="id",
            column_sort="time",
            column_kind=None,
            rolling_direction=-2,
            n_jobs=0,
        )

        self.assertListEqual(list(df["id"]), correct_indices)
        self.assertListEqual(list(df["a"].values), correct_values_a)
        self.assertListEqual(list(df["b"].values), correct_values_b)

    def test_stacked_rolling(self):
        first_class = pd.DataFrame(
            {"a": [1, 2, 3, 4], "b": [5, 6, 7, 8], "time": range(4)}
        )
        second_class = pd.DataFrame(
            {"a": [10, 11], "b": [12, 13], "time": range(20, 22)}
        )

        first_class["id"] = 1
        second_class["id"] = 2

        df_full = pd.concat([first_class, second_class], ignore_index=True)

        df_stacked = pd.concat(
            [
                df_full[["time", "id", "a"]].rename(columns={"a": "_value"}),
                df_full[["time", "id", "b"]].rename(columns={"b": "_value"}),
            ],
            ignore_index=True,
        )
        df_stacked["kind"] = ["a"] * 6 + ["b"] * 6

        """ df_stacked is
            time  id  _value kind
        0      0   1       1    a
        1      1   1       2    a
        2      2   1       3    a
        3      3   1       4    a
        4     20   2      10    a
        5     21   2      11    a
        6      0   1       5    b
        7      1   1       6    b
        8      2   1       7    b
        9      3   1       8    b
        10    20   2      12    b
        11    21   2      13    b
        """

        df = dataframe_functions.roll_time_series(
            df_stacked,
            column_id="id",
            column_sort="time",
            column_kind="kind",
            rolling_direction=-1,
            n_jobs=0,
        )

        correct_indices = (
            [(1, 0)] * 2 * 4
            + [(1, 1)] * 2 * 3
            + [(1, 2)] * 2 * 2
            + [(1, 3)] * 2 * 1
            + [(2, 20)] * 2 * 2
            + [(2, 21)] * 2 * 1
        )
        self.assertListEqual(list(df["id"].values), correct_indices)

        self.assertListEqual(list(df["kind"].values), ["a", "b"] * 13)
        self.assertListEqual(
            list(df["_value"].values),
            [
                1.0,
                5.0,
                2.0,
                6.0,
                3.0,
                7.0,
                4.0,
                8.0,
                2.0,
                6.0,
                3.0,
                7.0,
                4.0,
                8.0,
                3.0,
                7.0,
                4.0,
                8.0,
                4.0,
                8.0,
                10.0,
                12.0,
                11.0,
                13.0,
                11.0,
                13.0,
            ],
        )

    def test_dict_rolling(self):
        df_dict = {
            "a": pd.DataFrame(
                {"_value": [1, 2, 3, 4, 10, 11], "id": [1, 1, 1, 1, 2, 2]}
            ),
            "b": pd.DataFrame(
                {"_value": [5, 6, 7, 8, 12, 13], "id": [1, 1, 1, 1, 2, 2]}
            ),
        }
        df = dataframe_functions.roll_time_series(
            df_dict,
            column_id="id",
            column_sort=None,
            column_kind=None,
            rolling_direction=-1,
            n_jobs=0,
        )
        """ df is
        {a: _value  id
              1.0   1
              2.0   1
              3.0   1
              4.0   1
             10.0   2
             11.0   2,

         b: _value  id
               5.0   1
               6.0   1
               7.0   1
               8.0   1
              12.0   2
              13.0   2
         }
        """

        correct_indices = [
            (1, 0),
            (1, 0),
            (1, 0),
            (1, 0),
            (1, 1),
            (1, 1),
            (1, 1),
            (1, 2),
            (1, 2),
            (1, 3),
            (2, 0),
            (2, 0),
            (2, 1),
        ]
        self.assertListEqual(list(df["a"]["id"].values), correct_indices)

        self.assertListEqual(list(df["b"]["id"].values), correct_indices)

        self.assertListEqual(
            list(df["a"]["_value"].values),
            [1.0, 2.0, 3.0, 4.0, 2.0, 3.0, 4.0, 3.0, 4.0, 4.0, 10.0, 11.0, 11.0],
        )
        self.assertListEqual(
            list(df["b"]["_value"].values),
            [5.0, 6.0, 7.0, 8.0, 6.0, 7.0, 8.0, 7.0, 8.0, 8.0, 12.0, 13.0, 13.0],
        )

    def test_dict_rolling_maxshift_1(self):
        df_dict = {
            "a": pd.DataFrame(
                {"_value": [1, 2, 3, 4, 10, 11], "id": [1, 1, 1, 1, 2, 2]}
            ),
            "b": pd.DataFrame(
                {"_value": [5, 6, 7, 8, 12, 13], "id": [1, 1, 1, 1, 2, 2]}
            ),
        }
        df = dataframe_functions.roll_time_series(
            df_dict,
            column_id="id",
            column_sort=None,
            column_kind=None,
            rolling_direction=-1,
            max_timeshift=1,
            n_jobs=0,
        )
        """ df is
        {a: _value  id
              1.0   1
              2.0   1
              3.0   1
              4.0   1
             10.0   2
             11.0   2,

         b: _value  id
               5.0   1
               6.0   1
               7.0   1
               8.0   1
              12.0   2
              13.0   2
         }
        """

        correct_indices = [
            (1, 0),
            (1, 0),
            (1, 1),
            (1, 1),
            (1, 2),
            (1, 2),
            (1, 3),
            (2, 0),
            (2, 0),
            (2, 1),
        ]

        self.assertListEqual(list(df["a"]["id"].values), correct_indices)
        self.assertListEqual(list(df["b"]["id"].values), correct_indices)

        self.assertListEqual(
            list(df["a"]["_value"].values),
            [1.0, 2.0, 2.0, 3.0, 3.0, 4.0, 4.0, 10.0, 11.0, 11.0],
        )
        self.assertListEqual(
            list(df["b"]["_value"].values),
            [5.0, 6.0, 6.0, 7.0, 7.0, 8.0, 8.0, 12.0, 13.0, 13.0],
        )

    def test_order_rolling(self):

        first_class = pd.DataFrame({"x": [1, 2, 3, 4], "time": [1, 15, 132, 145]})
        second_class = pd.DataFrame({"x": [5, 6, 7], "time": [16, 133, 146]})

        first_class["initial_id"] = 1
        second_class["initial_id"] = 2
        df_full = pd.concat([first_class, second_class], ignore_index=True)

        # Do not show the warning on non-equidistant time
        with warning_free():
            window_size = 2

            df_rolled = dataframe_functions.roll_time_series(
                df_full,
                column_id="initial_id",
                column_sort="time",
                min_timeshift=window_size - 1,
                max_timeshift=window_size - 1,
            )

        """ df is
        {x: _value  id
              1.0   1
              2.0   1
              3.0   1
              4.0   1
              5.0   2
              6.0   2
              7.0   2,
         }
        """

        correct_indices = [
            (1, 15),
            (1, 15),
            (1, 132),
            (1, 132),
            (1, 145),
            (1, 145),
            (2, 133),
            (2, 133),
            (2, 146),
            (2, 146),
        ]

        self.assertListEqual(list(df_rolled["id"]), correct_indices)

    def test_warning_on_non_uniform_time_steps(self):
        with warnings.catch_warnings(record=True) as w:
            first_class = pd.DataFrame(
                {"a": [1, 2, 3, 4], "b": [5, 6, 7, 8], "time": [1, 2, 4, 5]}
            )
            second_class = pd.DataFrame(
                {"a": [10, 11], "b": [12, 13], "time": list(range(20, 22))}
            )

            first_class["id"] = 1
            second_class["id"] = 2

            df_full = pd.concat([first_class, second_class], ignore_index=True)

            dataframe_functions.roll_time_series(
                df_full,
                column_id="id",
                column_sort="time",
                column_kind=None,
                rolling_direction=1,
                n_jobs=0,
            )

            self.assertGreaterEqual(len(w), 1)
            self.assertIn(
                "Your time stamps are not uniformly sampled, which makes rolling "
                "nonsensical in some domains.",
                [str(warning.message) for warning in w],
            )

    def test_multicore_rolling(self):
        first_class = pd.DataFrame(
            {"a": [1, 2, 3, 4], "b": [5, 6, 7, 8], "time": range(4)}
        )
        second_class = pd.DataFrame(
            {"a": [10, 11], "b": [12, 13], "time": range(20, 22)}
        )

        first_class["id"] = 1
        second_class["id"] = 2

        df_full = pd.concat([first_class, second_class], ignore_index=True)

        """ df_full is
            a   b  time  id
        0   1   5     0   1
        1   2   6     1   1
        2   3   7     2   1
        3   4   8     3   1
        4  10  12    20   2
        5  11  13    21   2
        """
        correct_indices = [
            (1, 0),
            (1, 1),
            (1, 1),
            (1, 2),
            (1, 2),
            (1, 2),
            (1, 3),
            (1, 3),
            (1, 3),
            (1, 3),
            (2, 20),
            (2, 21),
            (2, 21),
        ]
        correct_values_a = [
            1.0,
            1.0,
            2.0,
            1.0,
            2.0,
            3.0,
            1.0,
            2.0,
            3.0,
            4.0,
            10.0,
            10.0,
            11.0,
        ]
        correct_values_b = [
            5.0,
            5.0,
            6.0,
            5.0,
            6.0,
            7.0,
            5.0,
            6.0,
            7.0,
            8.0,
            12.0,
            12.0,
            13.0,
        ]

        df = dataframe_functions.roll_time_series(
            df_full,
            column_id="id",
            column_sort="time",
            column_kind=None,
            rolling_direction=1,
        )

        self.assertListEqual(list(df["id"]), correct_indices)
        self.assertListEqual(list(df["a"].values), correct_values_a)
        self.assertListEqual(list(df["b"].values), correct_values_b)

        df = dataframe_functions.roll_time_series(
            df_full,
            column_id="id",
            column_sort="time",
            column_kind=None,
            rolling_direction=1,
            n_jobs=0,
        )

        self.assertListEqual(list(df["id"]), correct_indices)
        self.assertListEqual(list(df["a"].values), correct_values_a)
        self.assertListEqual(list(df["b"].values), correct_values_b)


class CheckForNanTestCase(TestCase):
    def test_all_columns(self):
        test_df = pd.DataFrame([[1, 2, 3], [4, 5, 6]], index=[0, 1])

        # should not raise an exception
        dataframe_functions.check_for_nans_in_columns(test_df)

        test_df = pd.DataFrame([[1, 2, 3], [4, np.nan, 6]], index=[0, 1])

        self.assertRaises(
            ValueError, dataframe_functions.check_for_nans_in_columns, test_df
        )

    def test_not_all_columns(self):
        test_df = pd.DataFrame(
            [[1, 2, 3], [4, np.nan, 6]], index=[0, 1], columns=["a", "b", "c"]
        )

        self.assertRaises(
            ValueError, dataframe_functions.check_for_nans_in_columns, test_df
        )
        self.assertRaises(
            ValueError,
            dataframe_functions.check_for_nans_in_columns,
            test_df,
            ["a", "b"],
        )
        self.assertRaises(
            ValueError, dataframe_functions.check_for_nans_in_columns, test_df, ["b"]
        )
        self.assertRaises(
            ValueError, dataframe_functions.check_for_nans_in_columns, test_df, "b"
        )
        self.assertRaises(
            ValueError,
            dataframe_functions.check_for_nans_in_columns,
            test_df,
            ["c", "b"],
        )

        dataframe_functions.check_for_nans_in_columns(test_df, columns=["a", "c"])
        dataframe_functions.check_for_nans_in_columns(test_df, columns="a")


class ImputeTestCase(TestCase):
    def test_impute_zero(self):
        df = pd.DataFrame([{"value": np.nan}])
        dataframe_functions.impute_dataframe_zero(df)
        self.assertEqual(list(df.value), [0])

        df = pd.DataFrame([{"value": np.inf}])
        dataframe_functions.impute_dataframe_zero(df)
        self.assertEqual(list(df.value), [0])

        df = pd.DataFrame([{"value": -np.inf}])
        dataframe_functions.impute_dataframe_zero(df)
        self.assertEqual(list(df.value), [0])

        df = pd.DataFrame(
            [{"value": -np.inf}, {"value": np.nan}, {"value": np.inf}, {"value": 1}]
        )
        dataframe_functions.impute_dataframe_zero(df)
        self.assertEqual(list(df.value), [0, 0, 0, 1])

        df = pd.DataFrame(
            [{"value": -np.inf}, {"value": np.nan}, {"value": np.inf}, {"value": 1}]
        )
        df = df.astype(np.float64)
        df = dataframe_functions.impute_dataframe_zero(df)
        self.assertEqual(list(df.value), [0, 0, 0, 1])

        df = pd.DataFrame(
            [{"value": -np.inf}, {"value": np.nan}, {"value": np.inf}, {"value": 1}]
        )
        df = df.astype(np.float32)
        df = dataframe_functions.impute_dataframe_zero(df)
        self.assertEqual(list(df.value), [0, 0, 0, 1])

        df = pd.DataFrame([])
        dataframe_functions.impute_dataframe_zero(df)

        self.assertEqual(len(df), 0)

    def test_toplevel_impute(self):
        df = pd.DataFrame(
            np.transpose([[0, 1, 2, np.nan], [1, np.inf, 2, 3], [1, -3, -np.inf, 3]]),
            columns=["value_a", "value_b", "value_c"],
        )

        dataframe_functions.impute(df)

        self.assertEqual(list(df.value_a), [0, 1, 2, 1])
        self.assertEqual(list(df.value_b), [1, 3, 2, 3])
        self.assertEqual(list(df.value_c), [1, -3, -3, 3])

        df = pd.DataFrame(
            np.transpose(
                [[0, 1, 2, np.nan], [1, np.inf, 2, np.nan], [np.nan, -3, -np.inf, 3]]
            ),
            columns=["value_a", "value_b", "value_c"],
        )
        df = df.astype(np.float64)
        dataframe_functions.impute(df)

        self.assertEqual(list(df.value_a), [0, 1, 2, 1])
        self.assertEqual(list(df.value_b), [1, 2, 2, 1.5])
        self.assertEqual(list(df.value_c), [0, -3, -3, 3])

        df = pd.DataFrame(
            np.transpose(
                [[0, 1, 2, np.nan], [1, np.inf, 2, 3], [np.inf, -3, -np.inf, 3]]
            ),
            columns=["value_a", "value_b", "value_c"],
        )
        df = df.astype(np.float32)
        dataframe_functions.impute(df)

        self.assertEqual(list(df.value_a), [0, 1, 2, 1])
        self.assertEqual(list(df.value_b), [1, 3, 2, 3])
        self.assertEqual(list(df.value_c), [3, -3, -3, 3])

        df = pd.DataFrame([])
        dataframe_functions.impute(df)

        self.assertEqual(len(df), 0)

    def test_impute_range(self):
        def get_df():
            return pd.DataFrame(
                np.transpose(
                    [[0, 1, 2, np.nan], [1, np.inf, 2, 3], [1, -3, -np.inf, 3]]
                ),
                columns=["value_a", "value_b", "value_c"],
            )

        # check if values are replaced correctly
        df = get_df()
        col_to_max = {"value_a": 200, "value_b": 200, "value_c": 200}
        col_to_min = {"value_a": -134, "value_b": -134, "value_c": -134}
        col_to_median = {"value_a": 55, "value_b": 55, "value_c": 55}
        dataframe_functions.impute_dataframe_range(
            df, col_to_max, col_to_min, col_to_median
        )
        self.assertEqual(list(df.value_a), [0, 1, 2, 55])
        self.assertEqual(list(df.value_b), [1, 200, 2, 3])
        self.assertEqual(list(df.value_c), [1, -3, -134, 3])

        # check for error if column key is missing
        df = get_df()
        col_to_max = {"value_a": 200, "value_b": 200, "value_c": 200}
        col_to_min = {"value_a": -134, "value_b": -134, "value_c": -134}
        col_to_median = {"value_a": 55, "value_c": 55}
        self.assertRaises(
            ValueError,
            dataframe_functions.impute_dataframe_range,
            df,
            col_to_max,
            col_to_min,
            col_to_median,
        )

        # check for no error if column key is too much
        col_to_max = {"value_a": 200, "value_b": 200, "value_c": 200}
        col_to_min = {"value_a": -134, "value_b": -134, "value_c": -134}
        col_to_median = {"value_a": 55, "value_b": 55, "value_c": 55, "value_d": 55}
        dataframe_functions.impute_dataframe_range(
            df, col_to_max, col_to_min, col_to_median
        )

        # check for error if replacement value is not finite
        df = get_df()
        col_to_max = {"value_a": 200, "value_b": np.nan, "value_c": 200}
        col_to_min = {"value_a": -134, "value_b": -134, "value_c": -134}
        col_to_median = {"value_a": 55, "value_b": 55, "value_c": 55}
        self.assertRaises(
            ValueError,
            dataframe_functions.impute_dataframe_range,
            df,
            col_to_max,
            col_to_min,
            col_to_median,
        )
        df = get_df()
        col_to_max = {"value_a": 200, "value_b": 200, "value_c": 200}
        col_to_min = {"value_a": -134, "value_b": -np.inf, "value_c": -134}
        col_to_median = {"value_a": 55, "value_b": 55, "value_c": 55}
        self.assertRaises(
            ValueError,
            dataframe_functions.impute_dataframe_range,
            df,
            col_to_max,
            col_to_min,
            col_to_median,
        )

        df = get_df()
        col_to_max = {"value_a": 200, "value_b": 200, "value_c": 200}
        col_to_min = {"value_a": -134, "value_b": -134, "value_c": -134}
        col_to_median = {"value_a": 55, "value_b": 55, "value_c": np.inf}
        self.assertRaises(
            ValueError,
            dataframe_functions.impute_dataframe_range,
            df,
            col_to_max,
            col_to_min,
            col_to_median,
        )

        df = pd.DataFrame([0, 1, 2, 3, 4], columns=["test"])
        col_dict = {"test": 0}
        dataframe_functions.impute_dataframe_range(df, col_dict, col_dict, col_dict)

        self.assertEqual(df.columns, ["test"])
        self.assertListEqual(list(df["test"].values), [0, 1, 2, 3, 4])

        df = pd.DataFrame([])
        dataframe_functions.impute_dataframe_range(df, {}, {}, {})

        self.assertEqual(len(df), 0)


class RestrictTestCase(TestCase):
    def test_restrict_dataframe(self):
        df = pd.DataFrame({"id": [1, 2, 3] * 2})

        df_restricted = dataframe_functions.restrict_input_to_index(df, "id", [2])
        self.assertEqual(list(df_restricted.id), [2, 2])

        df_restricted2 = dataframe_functions.restrict_input_to_index(
            df, "id", [1, 2, 3]
        )
        self.assertTrue(df_restricted2.equals(df))

    def test_restrict_dict(self):
        kind_to_df = {
            "a": pd.DataFrame({"id": [1, 2, 3]}),
            "b": pd.DataFrame({"id": [3, 4, 5]}),
        }

        kind_to_df_restricted = dataframe_functions.restrict_input_to_index(
            kind_to_df, "id", [3]
        )
        self.assertEqual(list(kind_to_df_restricted["a"].id), [3])
        self.assertEqual(list(kind_to_df_restricted["b"].id), [3])

        kind_to_df_restricted2 = dataframe_functions.restrict_input_to_index(
            kind_to_df, "id", [1, 2, 3, 4, 5]
        )
        self.assertTrue(kind_to_df_restricted2["a"].equals(kind_to_df["a"]))
        self.assertTrue(kind_to_df_restricted2["b"].equals(kind_to_df["b"]))

    def test_restrict_wrong(self):
        other_type = np.array([1, 2, 3])

        self.assertRaises(
            TypeError,
            dataframe_functions.restrict_input_to_index,
            other_type,
            "id",
            [1, 2, 3],
        )


class GetRangeValuesPerColumnTestCase(TestCase):
    def test_ignores_non_finite_values(self):
        df = pd.DataFrame([0, 1, 2, 3, np.nan, np.inf, -np.inf], columns=["value"])

        (
            col_to_max,
            col_to_min,
            col_to_median,
        ) = dataframe_functions.get_range_values_per_column(df)

        self.assertEqual(col_to_max, {"value": 3})
        self.assertEqual(col_to_min, {"value": 0})
        self.assertEqual(col_to_median, {"value": 1.5})

    def test_range_values_correct_with_even_length(self):
        df = pd.DataFrame([0, 1, 2, 3], columns=["value"])

        (
            col_to_max,
            col_to_min,
            col_to_median,
        ) = dataframe_functions.get_range_values_per_column(df)

        self.assertEqual(col_to_max, {"value": 3})
        self.assertEqual(col_to_min, {"value": 0})
        self.assertEqual(col_to_median, {"value": 1.5})

    def test_range_values_correct_with_uneven_length(self):
        df = pd.DataFrame([0, 1, 2], columns=["value"])

        (
            col_to_max,
            col_to_min,
            col_to_median,
        ) = dataframe_functions.get_range_values_per_column(df)

        self.assertEqual(col_to_max, {"value": 2})
        self.assertEqual(col_to_min, {"value": 0})
        self.assertEqual(col_to_median, {"value": 1})

    def test_no_finite_values_yields_0(self):
        df = pd.DataFrame([np.nan, np.inf, -np.inf], columns=["value"])

        with warnings.catch_warnings(record=True) as w:
            (
                col_to_max,
                col_to_min,
                col_to_median,
            ) = dataframe_functions.get_range_values_per_column(df)

            self.assertEqual(len(w), 1)
            self.assertEqual(
                str(w[0].message),
                "The columns ['value'] did not have any finite values. Filling with zeros.",
            )

        self.assertEqual(col_to_max, {"value": 0})
        self.assertEqual(col_to_min, {"value": 0})
        self.assertEqual(col_to_median, {"value": 0})


class MakeForecastingFrameTestCase(TestCase):
    def test_make_forecasting_frame_list(self):
        df, y = dataframe_functions.make_forecasting_frame(
            x=range(4), kind="test", max_timeshift=1, rolling_direction=1
        )
        expected_df = pd.DataFrame(
            {
                "id": [("id", 1), ("id", 2), ("id", 3)],
                "kind": ["test"] * 3,
                "value": [0, 1, 2],
                "time": [0, 1, 2],
            }
        )

        expected_y = pd.Series(
            data=[1, 2, 3], index=[("id", 1), ("id", 2), ("id", 3)], name="value"
        )
        assert_frame_equal(
            df.sort_index(axis=1).reset_index(drop=True), expected_df.sort_index(axis=1)
        )
        assert_series_equal(y, expected_y)

    def test_make_forecasting_frame_range(self):
        df, y = dataframe_functions.make_forecasting_frame(
            x=np.arange(4), kind="test", max_timeshift=1, rolling_direction=1
        )
        expected_df = pd.DataFrame(
            {
                "id": list(zip(["id"] * 3, np.arange(1, 4))),
                "kind": ["test"] * 3,
                "value": np.arange(3),
                "time": [0, 1, 2],
            }
        )
        expected_y = pd.Series(
            data=[1, 2, 3], index=[("id", 1), ("id", 2), ("id", 3)], name="value"
        )
        assert_frame_equal(
            df.sort_index(axis=1).reset_index(drop=True), expected_df.sort_index(axis=1)
        )
        assert_series_equal(y, expected_y)

    def test_make_forecasting_frame_pdSeries(self):

        t_index = pd.date_range("1/1/2011", periods=4, freq="H")
        df, y = dataframe_functions.make_forecasting_frame(
            x=pd.Series(data=range(4), index=t_index),
            kind="test",
            max_timeshift=1,
            rolling_direction=1,
        )

        time_shifts = pd.DatetimeIndex(
            ["2011-01-01 01:00:00", "2011-01-01 02:00:00", "2011-01-01 03:00:00"],
            freq="H",
        )
        expected_y = pd.Series(
            data=[1, 2, 3], index=zip(["id"] * 3, time_shifts), name="value"
        )
        expected_df = pd.DataFrame(
            {
                "id": list(
                    zip(
                        ["id"] * 3,
                        pd.DatetimeIndex(
                            [
                                "2011-01-01 01:00:00",
                                "2011-01-01 02:00:00",
                                "2011-01-01 03:00:00",
                            ]
                        ),
                    )
                ),
                "kind": ["test"] * 3,
                "value": [0, 1, 2],
                "time": pd.DatetimeIndex(
                    [
                        "2011-01-01 00:00:00",
                        "2011-01-01 01:00:00",
                        "2011-01-01 02:00:00",
                    ]
                ),
            }
        )
        assert_frame_equal(
            df.sort_index(axis=1).reset_index(drop=True), expected_df.sort_index(axis=1)
        )
        assert_series_equal(y, expected_y)

    def test_make_forecasting_frame_feature_extraction(self):
        t_index = pd.date_range("1/1/2011", periods=4, freq="H")
        df, y = dataframe_functions.make_forecasting_frame(
            x=pd.Series(data=range(4), index=t_index),
            kind="test",
            max_timeshift=1,
            rolling_direction=1,
        )

        extract_relevant_features(
            df,
            y,
            column_id="id",
            column_sort="time",
            column_value="value",
            default_fc_parameters=MinimalFCParameters(),
        )


class GetIDsTestCase(TestCase):
    def test_get_id__correct_DataFrame(self):
        df = pd.DataFrame({"_value": [1, 2, 3, 4, 10, 11], "id": [1, 1, 1, 1, 2, 2]})
        self.assertEqual(dataframe_functions.get_ids(df, "id"), {1, 2})

    def test_get_id__correct_dict(self):
        df_dict = {
            "a": pd.DataFrame(
                {"_value": [1, 2, 3, 4, 10, 11], "id": [1, 1, 1, 1, 2, 2]}
            ),
            "b": pd.DataFrame(
                {"_value": [5, 6, 7, 8, 12, 13], "id": [4, 4, 3, 3, 2, 2]}
            ),
        }
        self.assertEqual(dataframe_functions.get_ids(df_dict, "id"), {1, 2, 3, 4})

    def test_get_id_wrong(self):
        other_type = np.array([1, 2, 3])
        self.assertRaises(TypeError, dataframe_functions.get_ids, other_type, "id")


class AddSubIdTestCase(TestCase):
    def test_no_parameters(self):
        dataframe = pd.DataFrame({"value": [1, 2, 3, 4, 5, 6, 7, 8, 9]})
        extended_dataframe = dataframe_functions.add_sub_time_series_index(dataframe, 2)

        self.assertEqual(list(extended_dataframe["id"]), [0, 0, 1, 1, 2, 2, 3, 3, 4])
        assert_series_equal(dataframe["value"], extended_dataframe["value"])

    def test_id_parameters(self):
        dataframe = pd.DataFrame(
            {"value": [1, 2, 3, 4, 5, 6, 7, 8, 9], "id": [1, 1, 1, 1, 2, 2, 2, 2, 2]}
        )

        extended_dataframe = dataframe_functions.add_sub_time_series_index(
            dataframe, 2, column_id="id"
        )

        self.assertEqual(
            list(extended_dataframe["id"]),
            [(0, 1), (0, 1), (1, 1), (1, 1), (0, 2), (0, 2), (1, 2), (1, 2), (2, 2)],
        )
        assert_series_equal(dataframe["value"], extended_dataframe["value"])

    def test_kind_parameters(self):
        dataframe = pd.DataFrame(
            {
                "value": [1, 2, 3, 4, 5, 6, 7, 8, 9],
                "id": [1, 1, 1, 1, 2, 2, 2, 2, 2],
                "kind": [0, 1, 0, 1, 0, 1, 0, 1, 0],
            }
        )

        extended_dataframe = dataframe_functions.add_sub_time_series_index(
            dataframe, 2, column_id="id", column_kind="kind"
        ).sort_index()

        self.assertEqual(
            list(extended_dataframe["id"]),
            [(0, 1), (0, 1), (0, 1), (0, 1), (0, 2), (0, 2), (0, 2), (0, 2), (1, 2)],
        )
        assert_series_equal(dataframe["value"], extended_dataframe["value"])
        assert_series_equal(dataframe["kind"], extended_dataframe["kind"])

    def test_sort_parameters(self):
        dataframe = pd.DataFrame(
            {
                "value": [1, 2, 3, 4, 5, 6, 7, 8, 9],
                "id": [1, 1, 1, 1, 2, 2, 2, 2, 2],
                "kind": [0, 1, 0, 1, 0, 1, 0, 1, 0],
                "sort": [9, 8, 7, 6, 5, 4, 3, 2, 1],
            }
        )

        extended_dataframe = dataframe_functions.add_sub_time_series_index(
            dataframe, 2, column_id="id", column_kind="kind", column_sort="sort"
        )

        self.assertEqual(
            list(extended_dataframe["id"]),
            [(0, 2), (0, 2), (0, 2), (0, 2), (1, 2), (0, 1), (0, 1), (0, 1), (0, 1)],
        )
        self.assertEqual(list(extended_dataframe["value"]), [9, 8, 7, 6, 5, 4, 3, 2, 1])
        self.assertEqual(list(extended_dataframe["kind"]), [0, 1, 0, 1, 0, 1, 0, 1, 0])
        self.assertEqual(list(extended_dataframe["sort"]), [1, 2, 3, 4, 5, 6, 7, 8, 9])

    def test_dict_input(self):
        dataframe = pd.DataFrame(
            {"value": [1, 2, 3, 4, 5, 6, 7, 8, 9], "id": [1, 1, 1, 1, 2, 2, 2, 2, 2]}
        )

        extended_dataframe = dataframe_functions.add_sub_time_series_index(
            {"1": dataframe}, 2, column_id="id"
        )

        self.assertIn("1", extended_dataframe)

        extended_dataframe = extended_dataframe["1"]

        self.assertEqual(
            list(extended_dataframe["id"]),
            [(0, 1), (0, 1), (1, 1), (1, 1), (0, 2), (0, 2), (1, 2), (1, 2), (2, 2)],
        )
        assert_series_equal(dataframe["value"], extended_dataframe["value"])



import math
from unittest import TestCase
from unittest.mock import Mock

import numpy as np
import pandas as pd
from dask import dataframe as dd

from tests.fixtures import DataTestCase
from tsfresh.feature_extraction.data import (
    LongTsFrameAdapter,
    PartitionedTsData,
    TsDictAdapter,
    WideTsFrameAdapter,
    to_tsdata,
)
from tsfresh.utilities.distribution import MultiprocessingDistributor

TEST_DATA_EXPECTED_TUPLES = [
    (
        10,
        "a",
        pd.Series(
            [
                36,
                71,
                27,
                62,
                56,
                58,
                67,
                11,
                2,
                24,
                45,
                30,
                0,
                9,
                41,
                28,
                33,
                19,
                29,
                43,
            ],
            index=[10] * 20,
        ),
    ),
    (
        10,
        "b",
        pd.Series(
            [
                78,
                37,
                23,
                44,
                6,
                3,
                21,
                61,
                39,
                31,
                53,
                16,
                66,
                50,
                40,
                47,
                7,
                42,
                38,
                55,
            ],
            index=[10] * 20,
        ),
    ),
    (
        500,
        "a",
        pd.Series(
            [
                76,
                72,
                74,
                75,
                32,
                64,
                46,
                35,
                15,
                70,
                57,
                65,
                51,
                26,
                5,
                25,
                10,
                69,
                73,
                77,
            ],
            index=[500] * 20,
        ),
    ),
    (
        500,
        "b",
        pd.Series(
            [
                8,
                60,
                12,
                68,
                22,
                17,
                18,
                63,
                49,
                34,
                20,
                52,
                48,
                14,
                79,
                4,
                1,
                59,
                54,
                13,
            ],
            index=[500] * 20,
        ),
    ),
]

WIDE_TEST_DATA_EXPECTED_TUPLES = [
    (
        10,
        "a",
        pd.Series(
            [
                11,
                9,
                67,
                45,
                30,
                58,
                62,
                19,
                56,
                29,
                0,
                27,
                36,
                43,
                33,
                2,
                24,
                71,
                41,
                28,
            ],
            index=list(range(20)),
        ),
    ),
    (
        10,
        "b",
        pd.Series(
            [
                50,
                40,
                39,
                7,
                53,
                23,
                16,
                37,
                66,
                38,
                6,
                47,
                3,
                61,
                44,
                42,
                78,
                31,
                21,
                55,
            ],
            index=list(range(20)),
        ),
    ),
    (
        500,
        "a",
        pd.Series(
            [
                15,
                35,
                25,
                32,
                69,
                65,
                70,
                64,
                51,
                46,
                5,
                77,
                26,
                73,
                76,
                75,
                72,
                74,
                10,
                57,
            ],
            index=list(range(20, 40)),
        ),
    ),
    (
        500,
        "b",
        pd.Series(
            [
                4,
                14,
                68,
                22,
                18,
                52,
                54,
                60,
                79,
                12,
                49,
                63,
                8,
                59,
                1,
                13,
                20,
                17,
                48,
                34,
            ],
            index=list(range(20, 40)),
        ),
    ),
]


class DataAdapterTestCase(DataTestCase):
    def test_long_tsframe(self):
        df = self.create_test_data_sample()
        data = LongTsFrameAdapter(df, "id", "kind", "val", "sort")

        self.assert_tsdata(data, TEST_DATA_EXPECTED_TUPLES)

    def test_long_tsframe_no_value_column(self):
        df = self.create_test_data_sample()
        data = LongTsFrameAdapter(df, "id", "kind", None, "sort")

        self.assert_tsdata(data, TEST_DATA_EXPECTED_TUPLES)

    def test_wide_tsframe(self):
        df = self.create_test_data_sample_wide()
        data = WideTsFrameAdapter(df, "id", "sort")

        self.assert_tsdata(data, WIDE_TEST_DATA_EXPECTED_TUPLES)

    def test_wide_tsframe_without_sort(self):
        df = self.create_test_data_sample_wide()
        del df["sort"]
        data = WideTsFrameAdapter(df, "id")

        self.assert_tsdata(data, WIDE_TEST_DATA_EXPECTED_TUPLES)

    def test_dict_tsframe(self):
        df = {key: df for key, df in self.create_test_data_sample().groupby("kind")}
        data = TsDictAdapter(df, "id", "val", "sort")

        self.assert_tsdata(data, TEST_DATA_EXPECTED_TUPLES)

    def assert_tsdata(self, data, expected):
        self.assertEqual(len(data), len(expected))
        self.assertEqual(sum(1 for _ in data), len(data))
        self.assert_data_chunk_object_equal(data, expected)

    def assert_data_chunk_object_equal(self, result, expected):
        dic_result = {str(x[0]) + "_" + str(x[1]): x[2] for x in result}
        dic_expected = {str(x[0]) + "_" + str(x[1]): x[2] for x in expected}
        for k in dic_result.keys():
            pd.testing.assert_series_equal(
                dic_result[k], dic_expected[k], check_names=False
            )

    def test_simple_data_sample_two_timeseries(self):
        df = pd.DataFrame(
            {"id": [10] * 4, "kind": ["a"] * 2 + ["b"] * 2, "val": [36, 71, 78, 37]}
        )
        df.set_index("id", drop=False, inplace=True)
        df.index.name = None

        result = to_tsdata(df, "id", "kind", "val")
        expected = [
            (10, "a", pd.Series([36, 71], index=[10] * 2, name="val")),
            (10, "b", pd.Series([78, 37], index=[10] * 2, name="val")),
        ]
        self.assert_data_chunk_object_equal(result, expected)

    def test_simple_data_sample_four_timeseries(self):
        df = self.create_test_data_sample()
        df.index.name = None
        df.sort_values(by=["id", "kind", "sort"], inplace=True)

        result = to_tsdata(df, "id", "kind", "val", "sort")
        expected = TEST_DATA_EXPECTED_TUPLES

        self.assert_data_chunk_object_equal(result, expected)

    def test_with_dictionaries_two_rows(self):
        test_df = pd.DataFrame(
            [
                {"value": 2, "sort": 2, "id": "id_1"},
                {"value": 1, "sort": 1, "id": "id_1"},
            ]
        )
        test_dict = {"a": test_df, "b": test_df}

        result = to_tsdata(
            test_dict, column_id="id", column_value="value", column_sort="sort"
        )
        expected = [
            ("id_1", "a", pd.Series([1, 2], index=[1, 0], name="value")),
            ("id_1", "b", pd.Series([1, 2], index=[1, 0], name="value")),
        ]
        self.assert_data_chunk_object_equal(result, expected)

    def test_with_dictionaries_two_rows(self):
        test_df = pd.DataFrame([{"value": 1, "id": "id_1"}, {"value": 2, "id": "id_1"}])
        test_dict = {"a": test_df, "b": test_df}

        result = to_tsdata(test_dict, column_id="id", column_value="value")
        expected = [
            ("id_1", "a", pd.Series([1, 2], index=[0, 1], name="value")),
            ("id_1", "b", pd.Series([1, 2], index=[0, 1], name="value")),
        ]
        self.assert_data_chunk_object_equal(result, expected)

    def test_wide_dataframe_order_preserved_with_sort_column(self):
        """verifies that the order of the sort column from a wide time series container is preserved"""

        test_df = pd.DataFrame(
            {
                "id": ["a", "a", "b"],
                "v1": [3, 2, 1],
                "v2": [13, 12, 11],
                "sort": [103, 102, 101],
            }
        )

        result = to_tsdata(test_df, column_id="id", column_sort="sort")
        expected = [
            ("a", "v1", pd.Series([2, 3], index=[1, 0], name="v1")),
            ("a", "v2", pd.Series([12, 13], index=[1, 0], name="v2")),
            ("b", "v1", pd.Series([1], index=[2], name="v1")),
            ("b", "v2", pd.Series([11], index=[2], name="v2")),
        ]
        self.assert_data_chunk_object_equal(result, expected)

    def test_dask_dataframe_with_kind(self):
        test_df = dd.from_pandas(
            pd.DataFrame({"id": [1, 2], "kind": ["a", "a"], "value": [1, 2]}),
            npartitions=1,
        )

        result = to_tsdata(test_df, column_id="id", column_kind="kind")
        self.assertEqual(result.column_id, "id")
        self.assertEqual(result.column_kind, "kind")
        self.assertEqual(result.column_value, "value")

        def test_f(chunk):
            return pd.DataFrame(
                {"id": chunk[0], "variable": chunk[1], "value": chunk[2]}
            )

        return_f = result.apply(
            test_f, meta=(("id", "int"), ("variable", "int"), ("value", "int"))
        ).compute()
        pd.testing.assert_frame_equal(
            return_f.reset_index(drop=True),
            pd.DataFrame({"id": [1, 2], "variable": ["a", "a"], "value": [1.0, 2.0]}),
        )

    def test_dask_dataframe_without_kind(self):
        test_df = dd.from_pandas(
            pd.DataFrame({"id": [1, 2], "value_a": [1, 2], "value_b": [3, 4]}),
            npartitions=1,
        )

        result = to_tsdata(test_df, column_id="id")
        self.assertEqual(result.column_id, "id")

        def test_f(chunk):
            return pd.DataFrame(
                {"id": chunk[0], "variable": chunk[1], "value": chunk[2]}
            )

        return_f = result.apply(
            test_f, meta=(("id", "int"), ("variable", "int"), ("value", "int"))
        ).compute()
        pd.testing.assert_frame_equal(
            return_f.sort_values("value").reset_index(drop=True),
            pd.DataFrame(
                {
                    "id": [1, 2, 1, 2],
                    "variable": ["value_a", "value_a", "value_b", "value_b"],
                    "value": [1.0, 2.0, 3.0, 4.0],
                }
            ),
        )

        test_df = dd.from_pandas(
            pd.DataFrame(
                {"id": [1, 1], "sort": [2, 1], "value_a": [1, 2], "value_b": [3, 4]}
            ),
            npartitions=1,
        )

        result = to_tsdata(test_df, column_id="id", column_sort="sort")
        self.assertEqual(result.column_id, "id")

        def test_f(chunk):
            return pd.DataFrame(
                {"id": chunk[0], "variable": chunk[1], "value": chunk[2]}
            )

        return_f = result.apply(
            test_f, meta=(("id", "int"), ("variable", "int"), ("value", "int"))
        ).compute()

        pd.testing.assert_frame_equal(
            return_f.reset_index(drop=True),
            pd.DataFrame(
                {
                    "id": [1, 1, 1, 1],
                    "variable": ["value_a", "value_a", "value_b", "value_b"],
                    "value": [2.0, 1.0, 4.0, 3.0],
                }
            ),
        )

    def test_with_wrong_input(self):
        test_df = pd.DataFrame([{"id": 0, "kind": "a", "value": 3, "sort": np.nan}])
        self.assertRaises(ValueError, to_tsdata, test_df, "id", "kind", "value", "sort")

        test_df = pd.DataFrame([{"id": 0, "kind": "a", "value": 3, "sort": 1}])
        self.assertRaises(
            ValueError, to_tsdata, test_df, "strange_id", "kind", "value", "sort"
        )
        test_df = dd.from_pandas(test_df, npartitions=1)
        self.assertRaises(
            ValueError, to_tsdata, test_df, "strange_id", "kind", "value", "sort"
        )

        test_df = pd.DataFrame(
            [{"id": 0, "kind": "a", "value": 3, "value_2": 1, "sort": 1}]
        )
        self.assertRaises(
            ValueError, to_tsdata, test_df, "strange_id", "kind", None, "sort"
        )
        test_df = dd.from_pandas(test_df, npartitions=1)
        self.assertRaises(
            ValueError, to_tsdata, test_df, "strange_id", "kind", None, "sort"
        )

        test_df = pd.DataFrame([{"id": 0, "kind": "a", "value": 3, "sort": 1}])
        self.assertRaises(
            ValueError, to_tsdata, test_df, "id", "strange_kind", "value", "sort"
        )
        test_df = dd.from_pandas(test_df, npartitions=1)
        self.assertRaises(
            ValueError, to_tsdata, test_df, "id", "strange_kind", "value", "sort"
        )

        test_df = pd.DataFrame([{"id": np.nan, "kind": "a", "value": 3, "sort": 1}])
        self.assertRaises(ValueError, to_tsdata, test_df, "id", "kind", "value", "sort")

        test_df = pd.DataFrame([{"id": 0, "kind": np.nan, "value": 3, "sort": 1}])
        self.assertRaises(ValueError, to_tsdata, test_df, "id", "kind", "value", "sort")

        test_df = pd.DataFrame([{"id": 2}, {"id": 1}])
        test_dd = dd.from_pandas(test_df, npartitions=1)
        test_dict = {"a": test_df, "b": test_df}

        # column_id needs to be given
        self.assertRaises(ValueError, to_tsdata, test_df, None, "a", "b", None)
        self.assertRaises(ValueError, to_tsdata, test_dd, None, "a", "b", None)
        self.assertRaises(ValueError, to_tsdata, test_df, None, "a", "b", "a")
        self.assertRaises(ValueError, to_tsdata, test_dd, None, "a", "b", "a")
        self.assertRaises(ValueError, to_tsdata, test_dict, None, "a", "b", None)
        self.assertRaises(ValueError, to_tsdata, test_dict, None, "a", "b", "a")

        # If there are more than one column, the algorithm can not choose the correct column
        self.assertRaises(ValueError, to_tsdata, test_dict, "id", None, None, None)

        test_dict = {
            "a": pd.DataFrame([{"id": 2, "value_a": 3}, {"id": 1, "value_a": 4}]),
            "b": pd.DataFrame([{"id": 2}, {"id": 1}]),
        }

        # If there are more than one column, the algorithm can not choose the correct column
        self.assertRaises(ValueError, to_tsdata, test_dict, "id", None, None, None)

        test_df = pd.DataFrame([{"id": 0, "value": np.nan}])
        self.assertRaises(ValueError, to_tsdata, test_df, "id", None, "value", None)

        test_df = pd.DataFrame([{"id": 0, "value": np.nan}])
        self.assertRaises(ValueError, to_tsdata, test_df, None, None, "value", None)

        test_df = pd.DataFrame([{"id": 0, "a_": 3, "b": 5, "sort": 1}])
        self.assertRaises(ValueError, to_tsdata, test_df, "id", None, None, "sort")
        test_df = dd.from_pandas(test_df, npartitions=1)
        self.assertRaises(ValueError, to_tsdata, test_df, "id", None, None, "sort")

        test_df = pd.DataFrame([{"id": 0, "a__c": 3, "b": 5, "sort": 1}])
        self.assertRaises(ValueError, to_tsdata, test_df, "id", None, None, "sort")
        test_df = dd.from_pandas(test_df, npartitions=1)
        self.assertRaises(ValueError, to_tsdata, test_df, "id", None, None, "sort")

        test_df = pd.DataFrame([{"id": 0}])
        self.assertRaises(ValueError, to_tsdata, test_df, "id", None, None, None)
        test_df = dd.from_pandas(test_df, npartitions=1)
        self.assertRaises(ValueError, to_tsdata, test_df, "id", None, None, None)

        test_df = pd.DataFrame([{"id": 0, "sort": 0}])
        self.assertRaises(ValueError, to_tsdata, test_df, "id", None, None, "sort")
        test_df = dd.from_pandas(test_df, npartitions=1)
        self.assertRaises(ValueError, to_tsdata, test_df, "id", None, None, "sort")

        test_df = [1, 2, 3]
        self.assertRaises(ValueError, to_tsdata, test_df, "a", "b", "c", "d")


class PivotListTestCase(TestCase):
    def test_empty_list(self):
        mock_ts_data = Mock()
        mock_ts_data.df_id_type = str

        return_df = PartitionedTsData.pivot(mock_ts_data, [])

        self.assertEqual(len(return_df), 0)
        self.assertEqual(len(return_df.index), 0)
        self.assertEqual(len(return_df.columns), 0)

    def test_different_input(self):
        mock_ts_data = Mock()
        mock_ts_data.df_id_type = str

        input_list = [
            ("a", "b", 1),
            ("a", "c", 2),
            ("A", "b", 3),
            ("A", "c", 4),
            ("X", "Y", 5),
        ]
        return_df = PartitionedTsData.pivot(mock_ts_data, input_list)

        self.assertEqual(len(return_df), 3)
        self.assertEqual(set(return_df.index), {"a", "A", "X"})
        self.assertEqual(set(return_df.columns), {"b", "c", "Y"})

        self.assertEqual(return_df.loc["a", "b"], 1)
        self.assertEqual(return_df.loc["a", "c"], 2)
        self.assertEqual(return_df.loc["A", "b"], 3)
        self.assertEqual(return_df.loc["A", "c"], 4)
        self.assertEqual(return_df.loc["X", "Y"], 5)

    def test_long_input(self):
        mock_ts_data = Mock()
        mock_ts_data.df_id_type = str

        input_list = []
        for i in range(100):
            for j in range(100):
                input_list.append((i, j, i * j))

        return_df = PartitionedTsData.pivot(mock_ts_data, input_list)

        self.assertEqual(len(return_df), 100)
        self.assertEqual(len(return_df.columns), 100)
        # every cell should be filled
        self.assertEqual(np.sum(np.sum(np.isnan(return_df))), 0)
        # test if all entries are there
        self.assertEqual(return_df.sum().sum(), 24502500)



# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016
import numpy as np
import pandas as pd
import pytest
from statsmodels.stats.multitest import multipletests


@pytest.mark.parametrize(
    "p_value, ind, fdr, expected",
    [
        ([0, 0, 0], True, 0.10, [True, True, True]),
        ([0, 0, 0], False, 0.10, [True, True, True]),
        ([0.1, 0.15, 0.2, 0], True, 0.20, [True, True, True, True]),
        ([0.1, 0.15, 0.2, 0], False, 0.20, [False, False, False, True]),
        ([0.1, 0.1, 0.05], True, 0.20, [True, True, True]),
        ([0.1, 0.11, 0.05], False, 0.20, [False, False, False]),
        ([0.1, 0.1, 0.05], False, 0.20, [True, True, True]),
        (
            [0.00356, 0.01042, 0.01208, 0.02155, 0.03329, 0.11542],
            True,
            0.05,
            [True, True, True, True, True, False],
        ),
        (
            [0.00356, 0.01042, 0.01208, 0.02155, 0.03329, 0.11542],
            False,
            0.05,
            [False, False, False, False, False, False],
        ),
        ([0.11, 0.001, 0.05], False, 0.20, [False, True, True]),
    ],
)
def test_fdr_control(p_value, ind, fdr, expected):
    df = pd.DataFrame({"p_value": p_value})
    method = "fdr_bh" if ind else "fdr_by"
    df["relevant"] = multipletests(pvals=df.p_value, alpha=fdr, method=method)[0]
    result = df["relevant"].values
    expected = np.array(expected)
    assert (result == expected).all()



# -*- coding: utf-8 -*-
# This file as well as the whole tsfresh package are licenced under the MIT licence (see the LICENCE.txt)
# Maximilian Christ (maximilianchrist.com), Blue Yonder Gmbh, 2016

import os
import shutil
import tempfile
from unittest import TestCase

import pandas as pd
from sklearn.pipeline import Pipeline

from tests.fixtures import warning_free
from tsfresh.examples.robot_execution_failures import (
    download_robot_execution_failures,
    load_robot_execution_failures,
)
from tsfresh.transformers import RelevantFeatureAugmenter


class FullPipelineTestCase_robot_failures(TestCase):
    def setUp(self):
        self.temporary_folder = tempfile.mkdtemp()
        temporary_file = os.path.join(self.temporary_folder, "data")

        download_robot_execution_failures(file_name=temporary_file)
        self.timeseries, self.y = load_robot_execution_failures(
            file_name=temporary_file
        )
        self.df = pd.DataFrame(index=self.timeseries.id.unique())

        # shrink the time series for this test
        self.timeseries = self.timeseries[["id", "time", "F_x"]]

    def tearDown(self):
        shutil.rmtree(self.temporary_folder)

    def test_relevant_extraction(self):
        self.assertGreater(len(self.y), 0)
        self.assertGreater(len(self.df), 0)
        self.assertGreater(len(self.timeseries), 0)

        relevant_augmenter = RelevantFeatureAugmenter(
            column_id="id", column_sort="time"
        )
        relevant_augmenter.set_timeseries_container(self.timeseries)

        pipe = Pipeline([("relevant_augmenter", relevant_augmenter)])

        with warning_free():
            pipe.fit(self.df, self.y)
        extracted_features = pipe.transform(self.df)

        some_expected_features = {
            "F_x__abs_energy",
            "F_x__absolute_sum_of_changes",
            "F_x__autocorrelation__lag_1",
            "F_x__binned_entropy__max_bins_10",
            "F_x__count_above_mean",
            "F_x__longest_strike_above_mean",
            "F_x__maximum",
            "F_x__mean_abs_change",
            "F_x__minimum",
            "F_x__quantile__q_0.1",
            "F_x__range_count__max_1__min_-1",
            "F_x__spkt_welch_density__coeff_2",
            "F_x__standard_deviation",
            "F_x__value_count__value_0",
            "F_x__variance",
            "F_x__variance_larger_than_standard_deviation",
        }

        self.assertGreaterEqual(set(extracted_features.columns), some_expected_features)
        self.assertGreater(len(extracted_features), 0)
